@@ -24,9 +24,9 @@ private:
     std::vector<bool> &in_is_na
   ) : address_v4(in_address_v4), address_v6(in_address_v6), is_ipv6(in_is_ipv6), is_na(in_is_na) { };
 
-  Rcpp::LogicalVector isTrue(
-      const std::function<bool(const asio::ip::address_v4&)>& decide_fn_v4,
-      const std::function<bool(const asio::ip::address_v6&)>& decide_fn_v6
+  Rcpp::LogicalVector checkCondition(
+      const std::function<bool(const asio::ip::address_v4&)>& condition_v4,
+      const std::function<bool(const asio::ip::address_v6&)>& condition_v6
   ) const;
 
   IpAddressVector map6to4(
@@ -63,7 +63,7 @@ public:
   static IpAddressVector createHostmask(Rcpp::IntegerVector prefix_length, Rcpp::LogicalVector is_ipv6);
 
   // Warn about invalid input
-  static void warnInvalidInput(unsigned int index, const std::string &input, const std::string &reason = "");
+  static void warnOnRow(unsigned int index, const std::string &input, const std::string &reason = "");
 
 
   /*----------*
@@ -107,6 +107,7 @@ public:
    *-----------------------*/
   Rcpp::LogicalVector isWithin(const IpNetworkVector &network) const;
   Rcpp::LogicalVector isWithinAny(const IpNetworkVector &network) const;
+  Rcpp::IntegerVector prefixFromMask() const;
 
 
   /*----------------------*
