@@ -23,15 +23,15 @@ test_that("missing values work", {
 })
 
 test_that("invalid inputs are caught", {
-  expect_warning(ip_address("1.2.3.4/5"), "Invalid value")
-  expect_warning(ip_address("abc"), "Invalid value")
-  expect_warning(ip_address("1.2.3.256"), "Invalid value")
-  expect_warning(ip_address("1.2.3.-1"), "Invalid value")
-  expect_warning(ip_address("1.2.3.4.5"), "Invalid value")
+  expect_warning(ip_address("1.2.3.4/5"))
+  expect_warning(ip_address("abc"))
+  expect_warning(ip_address("1.2.3.256"))
+  expect_warning(ip_address("1.2.3.-1"))
+  expect_warning(ip_address("1.2.3.4.5"))
 
   # Windows accepts IP addresses in alternative formats: https://superuser.com/a/486936
   skip_on_os("windows")
-  expect_warning(ip_address("1.2.3"), "Invalid value")
+  expect_warning(ip_address("1.2.3"))
 })
 
 test_that("equality operations work", {
